<?php
/**
 * @version		1.3.0
 * @package		Joomla
 * @subpackage	Helpdesk Pro
 * @author  Tuan Pham Ngoc
 * @copyright	Copyright (C) 2013 Ossolution Team
 * @license		GNU/GPL, see LICENSE.php
 */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

OSFactory::loadLibrary('modellist');

class HelpdeskproModelTickets extends OSModelList
{

	function __construct($config)
	{
		$config['title_field'] = array('a.id', 'a.name', 'a.email', 'c.username', 'a.subject', 'a.message');
		$config['default_ordering'] = ' a.created_date';
		$config['state_vars'] = array(
			'category_id' => array(0, 'int', true), 
			'manager_id' => array(0, 'int', true), 
			'status_id' => array(-1, 'int', true), 
			'priority_id' => array(0, 'int', true), 
			'filter_order' => array('a.modified_date', 'string', true), 
			'filter_order_Dir' => array('DESC', 'cmd', true), 
			'published' => array(-1, 'int', true), 
			'label_id' => array(0, 'int', true));
		
		parent::__construct($config);
	}

	function _buildQuery()
	{
		$where = $this->_buildContentWhere();
		$orderby = $this->_buildContentOrderBy();
		$query = 'SELECT a.*, b.title AS category_title, c.username AS username, d.title AS label_title, d.color_code FROM #__helpdeskpro_tickets  AS a ' .
			 ' LEFT JOIN #__helpdeskpro_categories AS b ' . ' ON a.category_id= b.id ' . ' LEFT JOIN #__users AS c ' . ' ON a.user_id = c.id ' .
			 ' LEFT JOIN #__helpdeskpro_labels AS d ' . ' ON a.label_id = d.id ' . $where . $orderby;		
		return $query;
		
		
	}

	function getTotal()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$where = $this->_buildContentWhere();
			$sql = 'SELECT COUNT(a.id) FROM #__helpdeskpro_tickets  AS a ' . ' LEFT JOIN #__users AS c ' . ' ON a.user_id = c.id ' . $where;
			$this->_db->setQuery($sql);
			$this->_total = $this->_db->loadResult();
		}
		
		return $this->_total;
	}

	function _buildContentWhereArray()
	{
		$app = JFactory::getApplication();
		$user = JFactory::getUser();
		$config = HelpdeskProHelper::getConfig();
		$managedCategoryIds = HelpdeskProHelper::getTicketCategoryIds($user->get('username'));
		//$where = parent::_buildContentWhereArray();
		$where = array();		
		$state = $this->getState();
		if ($state->category_id)
		{
			$where[] = ' a.category_id = ' . $state->category_id;
		}
		
		if ($state->status_id)
		{
			if ($state->status_id == -1)
			{
				if (!$user->authorise('core.admin'))
				{
					if (count($managedCategoryIds))
					{
						//Show open and pending tickets to managers by default
						$where[] = ' (a.status_id=' . $config->new_ticket_status_id . ' OR a.status_id=' .
							 $config->ticket_status_when_customer_add_comment . ') ';
					}
					else
					{
						//Show open tickets and require feedback tickets to customers
						$where[] = " a.status_id != $config->closed_ticket_status ";
					}
				}
				else
				{
					//Show open and pending tickets to managers by default
					$where[] = ' (a.status_id=' . $config->new_ticket_status_id . ' OR a.status_id=' .
						 $config->ticket_status_when_customer_add_comment . ') ';
				}
			}
			else
			{
				$where[] = ' a.status_id = ' . $state->status_id;
			}
		}
		
		if ($state->priority_id)
		{
			$where[] = ' a.priority_id = ' . $state->priority_id;
		}
		
		if ($state->label_id)
		{
			$where[] = ' a.label_id = ' . $state->label_id;
		}
		
		if (!$user->authorise('core.admin'))
		{ 			
			if ($managedCategoryIds)
			{ //He is ticket managers, so all tickets belong to him
				$where[] = ' a.category_id IN (' . implode(',', $managedCategoryIds) . ')';
			}
			else
			{
				$db = $this->getDbo();
				$userId = $user->get('id');
				$sql = 'SELECT COUNT(*) FROM #__helpdeskpro_tickets WHERE staff_id=' . $userId;
				$db->setQuery($sql);
				$total = (int) $db->loadResult();							
				if ($total)
				{
					$where[] = ' a.staff_id=' . $userId;
				}
				else
				{
					$email = $user->get('email');
					$where[] = "(a.user_id=$userId OR a.email='$email') ";
				}
			}
		}
		
		if ($state->filter_state == 'P')
		{
			$where[] = ' a.published=1 ';
		}			
		elseif ($state->filter_state == 'U')
		{
			$where[] = ' a.published = 0';
		}				
		if ($state->search)
		{
			$search = $db->Quote('%' . $db->escape($state->search, true) . '%', false);			
			$whereOr = array();
			foreach ($this->titleField as $titleField)
			{
				$whereOr[] = " LOWER($titleField) LIKE " . $search;
			}
			$whereOr[] = " a.id IN (SELECT ticket_id FROM #__helpdeskpro_messages WHERE LOWER(`message`) LIKE $search)";
			$where[] = ' (' . implode(' OR ', $whereOr) . ') ';						
		}
		
		return $where;
	}
}